
-- Project: fifo
--
-- Controls two external 7-segment displays.
--
-- !!!! this is for the Digilent board !!!!

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

entity display_drive is
  port 
   (
    clk          : in   std_logic;
    fifo_input   : in   std_logic_vector(2 downto 0);
    tag          : in   std_logic;
    end_game     : in   std_logic;
    fifo_output  : in   std_logic_vector(2 downto 0);
    fifo_level   : in   std_logic_vector(3 downto 0);
    point_count  : in   std_logic_vector(3 downto 0);
    --
    seven_seg_1  : out  std_logic_vector(7 downto 0);
    seven_seg_2  : out  std_logic_vector(7 downto 0);
    seven_seg_3  : out  std_logic_vector(7 downto 0);
    seven_seg_4  : out  std_logic_vector(7 downto 0);
    en1          : out  std_logic;
    en2          : out  std_logic;
    en3          : out  std_logic;
    en4          : out  std_logic 
   );
end entity;

architecture Behavioral of display_drive is

  component seg_encode
  port 
   (
    hex_in     : in   std_logic_vector(3 downto 0);
    seven_seg  : out  std_logic_vector(7 downto 0)
   );
  end component;

  signal blink_cnt     : unsigned(27 downto 0);
  signal encode3_input : std_logic_vector(3 downto 0);
  signal ms_cnt       : unsigned(16 downto 0);
  signal seg_sel      : unsigned(1 downto 0);
  signal seg_1        : std_logic_vector(7 downto 0);
  signal seg_2        : std_logic_vector(7 downto 0);
  signal seg_3        : std_logic_vector(7 downto 0);
  signal seg_4        : std_logic_vector(7 downto 0);

begin

  timer : process(clk)
  begin
     if rising_edge(clk) then
        if (blink_cnt = X"0000000") then
           blink_cnt <= X"1000000";
        else
           blink_cnt <= blink_cnt - 1;
        end if;
        --
        if (end_game = '1') then
           seg_1 <= (others => blink_cnt(23));
           seg_2 <= (others => blink_cnt(23));
        elsif (tag = '1') then
           seg_1 <= (others => blink_cnt(23));
        else
           case(fifo_input) is 
             when "000" => seg_1 <= "11111110";
             when "001" => seg_1 <= "11111101";
             when "010" => seg_1 <= "11111011";
             when "011" => seg_1 <= "11110111";
             when "100" => seg_1 <= "11101111";
             when "101" => seg_1 <= "11011111";
             when "110" => seg_1 <= "10111111";
             when "111" => seg_1 <= "01111111";
             when others => null;
           end case;
           --
           case(fifo_output) is 
             when "000" => seg_2 <= "11111110";
             when "001" => seg_2 <= "11111101";
             when "010" => seg_2 <= "11111011";
             when "011" => seg_2 <= "11110111";
             when "100" => seg_2 <= "11101111";
             when "101" => seg_2 <= "11011111";
             when "110" => seg_2 <= "10111111";
             when "111" => seg_2 <= "01111111";
             when others => null;
           end case;
        end if;
     end if;
  end process;
   
  encode3_input <= X"0" when end_game = '1' else fifo_level;
  
  seg_encode_3 : seg_encode
  port map
   (
    hex_in     => encode3_input,
    seven_seg  => seg_3
   );
   
  seg_encode_4 : seg_encode
  port map
   (
    hex_in     => point_count,
    seven_seg  => seg_4
   );
   
  -- Cycle through the four segments.
  main_proc : process(clk)
  begin
    if rising_edge(clk) then
      ms_cnt <= ms_cnt + 1; -- ~~1.3ms at 100MHz
      --
      if (ms_cnt = '0' & X"0000") then
        seg_sel <= seg_sel + 1;
      end if;
    end if;
  end process;

  -- 7-segment enables are low-active
  en1 <= '0' when seg_sel = "00" else '1';
  en2 <= '0' when seg_sel = "01" else '1';
  en3 <= '0' when seg_sel = "10" else '1';
  en4 <= '0' when seg_sel = "11" else '1';

  seven_seg_1 <= seg_1 when seg_sel = "00" else 
                 seg_2 when seg_sel = "01" else 
                 seg_3 when seg_sel = "10" else 
                 seg_4;

  seven_seg_2 <= X"00";
  seven_seg_3 <= X"00";
  seven_seg_4 <= X"00";
                 
end architecture Behavioral;

