
-- Project: memories
--
-- Stores hex values in memory, which can then be displayed.

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

entity memories is
  port 
   (
    clk          : in   std_logic;
    pb_1         : in   std_logic;
    pb_2         : in   std_logic;
    en_wr        : in   std_logic;
    sw           : in   std_logic_vector(7 downto 0);
    --
    seven_seg_1  : out  std_logic_vector(7 downto 0);
    seven_seg_2  : out  std_logic_vector(7 downto 0);
    seven_seg_3  : out  std_logic_vector(7 downto 0);
    seven_seg_4  : out  std_logic_vector(7 downto 0);
    en1          : out  std_logic;
    en2          : out  std_logic;
    en3          : out  std_logic;
    en4          : out  std_logic;
    led_1        : out  std_logic;
    led_2        : out  std_logic;
    -- unused LEDs
    led       : out  std_logic_vector(9 downto 2)
   );
end entity;

architecture Behavioral of memories is

   -- component declarations
   
  component debounce is
  port 
   (
    clk          : in   std_logic;
    pb_in        : in   std_logic;
    --
    pb_debounced : out  std_logic
   );
   end component;

  component memory_8x8
  port
   (
    clk        : in  std_logic;
    dat_in     : in  std_logic_vector (7 downto 0);
    adr        : in  std_logic_vector (2 downto 0);
    wr         : in  std_logic;
    dat_out    : out std_logic_vector (7 downto 0)
   );
  end component;

  component seg_drive
  port 
   (
    clk          : in   std_logic;
    mem_dat_1    : in   std_logic_vector(7 downto 0);
    mem_dat_2    : in   std_logic_vector(7 downto 0);
    --
    seven_seg_1  : out  std_logic_vector(7 downto 0);
    seven_seg_2  : out  std_logic_vector(7 downto 0);
    seven_seg_3  : out  std_logic_vector(7 downto 0);
    seven_seg_4  : out  std_logic_vector(7 downto 0);
    en1          : out  std_logic;
    en2          : out  std_logic;
    en3          : out  std_logic;
    en4          : out  std_logic 
   );
  end component;
  
  -- signal declarations
  signal pb_debncd_1    : std_logic;
  signal pb_debncd_1_d1 : std_logic;
  signal pb_debncd_2    : std_logic;
  signal pb_debncd_2_d1 : std_logic;
  signal tog            : std_logic;
  signal pb1_pulse      : std_logic;
  signal mem_adr        : unsigned(2 downto 0);
  signal byte_in        : std_logic_vector(7 downto 0);
  signal mem_adr_std    : std_logic_vector(2 downto 0);
  signal mem_wr_1       : std_logic;
  signal mem_wr_2       : std_logic;
  signal mem_dat_out_1  : std_logic_vector(7 downto 0);
  signal mem_dat_out_2  : std_logic_vector(7 downto 0);
   
begin

  debounce_1 : debounce
  port map
   (
    clk          => clk,         --in   std_logic;
    pb_in        => pb_1,        --in   std_logic;
    --
    pb_debounced => pb_debncd_1  --out  std_logic
   );

  debounce_2 : debounce
  port map
   (
    clk          => clk,         --in   std_logic;
    pb_in        => pb_2,        --in   std_logic;
    --
    pb_debounced => pb_debncd_2  --out  std_logic
   );

  main_proc : process(clk)
  begin
    if rising_edge(clk) then
      pb_debncd_2_d1 <= pb_debncd_2;
      if (pb_debncd_2 = '0' and pb_debncd_2_d1 = '1') then 
         tog <= not tog;
      end if;
      --
      pb_debncd_1_d1 <= pb_debncd_1;
      if (pb_debncd_1 = '0' and pb_debncd_1_d1 = '1') then 
         mem_adr <= mem_adr + 1;
         pb1_pulse <= '1';
      else
         pb1_pulse <= '0';
      end if;
    end if;
  end process;

  led_1 <= en_wr and not tog;
  led_2 <= en_wr and     tog;

  byte_in     <= sw(7 downto 0);
  mem_wr_1    <= (not tog) and en_wr and pb1_pulse;
  mem_wr_2    <=      tog  and en_wr and pb1_pulse;
  mem_adr_std <= std_logic_vector(mem_adr);
  
  memory_8x8_1 : memory_8x8
  port map
   (
    clk      => clk,           --in  std_logic;
    dat_in   => byte_in,       --in  std_logic_vector (7 downto 0);
    adr      => mem_adr_std,   --in  std_logic_vector (2 downto 0);
    wr       => mem_wr_1,      --in  std_logic;
    dat_out  => mem_dat_out_1  --out std_logic_vector (7 downto 0)
   );

  memory_8x8_2 : memory_8x8
  port map
   (
    clk      => clk,           --in  std_logic;
    dat_in   => byte_in,       --in  std_logic_vector (7 downto 0);
    adr      => mem_adr_std,   --in  std_logic_vector (2 downto 0);
    wr       => mem_wr_2,      --in  std_logic;
    dat_out  => mem_dat_out_2  --out std_logic_vector (7 downto 0)
   );

  seg_drive_i : seg_drive
  port map
   (
    clk          => clk,            --in   std_logic;
    mem_dat_1    => mem_dat_out_1,  --in   std_logic_vector(7 downto 0);
    mem_dat_2    => mem_dat_out_2,  --in   std_logic_vector(7 downto 0);
    --                              
    seven_seg_1  => seven_seg_1,    --out  std_logic_vector(7 downto 0);
    seven_seg_2  => seven_seg_2,    --out  std_logic_vector(7 downto 0);
    seven_seg_3  => seven_seg_3,    --out  std_logic_vector(7 downto 0);
    seven_seg_4  => seven_seg_4,    --out  std_logic_vector(7 downto 0);
    en1          => en1,            --out  std_logic;
    en2          => en2,            --out  std_logic;
    en3          => en3,            --out  std_logic;
    en4          => en4             --out  std_logic 
   );
   
end architecture Behavioral;

