
-- Project: fifo_game
--
-- game to catch delayed tagged samples in a sequences of numbers.

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

use work.projects_pkg.all;

entity fifo_game is
  port 
   (
    clk          : in  std_logic;
    pb_1         : in  std_logic;
    pb_2         : in  std_logic;
    speed_select : in  std_logic;
    --
    seven_seg_1  : out std_logic_vector(7 downto 0);
    seven_seg_2  : out std_logic_vector(7 downto 0);
    seven_seg_3  : out std_logic_vector(7 downto 0);
    seven_seg_4  : out std_logic_vector(7 downto 0);
    en1          : out std_logic;
    en2          : out std_logic;
    en3          : out std_logic;
    en4          : out std_logic;
    -- unused LEDs
    led          : out  std_logic_vector(9 downto 0)
   );
end entity;

architecture Behavioral of fifo_game is

   -- component declarations
  component debounce is
  port 
   (
    clk          : in   std_logic;
    pb_in        : in   std_logic;
    --
    pb_debounced : out  std_logic
   );
   end component;

  component pr_gen is
  port 
   (
    clk          : in  std_logic;
    start        : in  std_logic;
    speed_select : in  std_logic;
    prbs         : out std_logic_vector(2 downto 0);
    tag          : out std_logic; --active for one full cycle time
    step         : out std_logic
   );
   end component;

  component fifo is
  port 
   (
    clk          : in  std_logic;
    start        : in  std_logic;
    prbs_in      : in  std_logic_vector(2 downto 0);
    tag_in       : in  std_logic;
    wr           : in  std_logic;
    --
    rd           : in  std_logic;
--  level        : out std_logic_vector(3 downto 0);
    level        : out std_logic_vector(4 downto 0);
    prbs_out     : out std_logic_vector(2 downto 0);
    tag_out      : out std_logic
   );
   end component;
   
  component display_drive is
  port 
   (
    clk          : in   std_logic;
    fifo_input   : in   std_logic_vector(2 downto 0);
    tag          : in   std_logic;
    end_game     : in   std_logic;
    fifo_output  : in   std_logic_vector(2 downto 0);
--  fifo_level   : in   std_logic_vector(3 downto 0);
    fifo_level   : in   std_logic_vector(4 downto 0);
    point_count  : in   std_logic_vector(3 downto 0);
    --           
    seven_seg_1  : out  std_logic_vector(7 downto 0);
    seven_seg_2  : out  std_logic_vector(7 downto 0);
    seven_seg_3  : out  std_logic_vector(7 downto 0);
    seven_seg_4  : out  std_logic_vector(7 downto 0);
    en1          : out  std_logic;
    en2          : out  std_logic;
    en3          : out  std_logic;
    en4          : out  std_logic 
   );
  end component;
  
  type fifo_state_type is (
                           idle,
                           incr,
                           decr,
                           pop,
                           full,
                           clear
                          );
  signal fifo_state : fifo_state_type;                          
                           
  signal pb_debncd_1      : std_logic;
  signal pb_debncd_1_d1   : std_logic;
  signal pb_debncd_2      : std_logic;
  signal pb_debncd_2_d1   : std_logic;
  signal pb_1_pls         : std_logic;
  signal pb_2_pls         : std_logic;
  signal pb_debncd_1_pol  : std_logic;
  signal pb_debncd_2_pol  : std_logic;
  signal start            : std_logic;
  signal next_samp        : std_logic;
  signal prbs             : std_logic_vector(2 downto 0);
  signal affirm           : std_logic;
  signal next_pls         : std_logic;
  signal affirm_pls       : std_logic;
  signal step             : std_logic;
  signal tag              : std_logic;
  signal tag_d            : std_logic;
  signal tag_out          : std_logic;
  signal rd_fifo          : std_logic;
--signal level            : std_logic_vector(3 downto 0);
  signal level            : std_logic_vector(4 downto 0);
  signal prbs_out         : std_logic_vector(2 downto 0);
  signal point_count      : unsigned(3 downto 0);
  signal point_count_std  : std_logic_vector(3 downto 0);
  signal point_cnt_clr    : std_logic;
  signal point_cnt_incr   : std_logic;
  signal point_cnt_decr   : std_logic;
  signal end_game         : std_logic;
  signal affirm_hold      : std_logic;
   
begin

  debounce_1 : debounce
  port map
   (
    clk          => clk,         --in   std_logic;
    pb_in        => pb_1,        --in   std_logic;
    --
    pb_debounced => pb_debncd_1  --out  std_logic
   );

  debounce_2 : debounce
  port map
   (
    clk          => clk,         --in   std_logic;
    pb_in        => pb_2,        --in   std_logic;
    --
    pb_debounced => pb_debncd_2  --out  std_logic
   );

  pb_debncd_1_pol <= polarity XOR pb_debncd_1;
  pb_debncd_2_pol <= polarity XOR pb_debncd_2;
  
  pb_proc : process(clk)
  begin
    if rising_edge(clk) then
      pb_debncd_1_d1 <= pb_debncd_1_pol;
      pb_1_pls       <= pb_debncd_1_pol and NOT pb_debncd_1_d1;
      --
      pb_debncd_2_d1 <= pb_debncd_2_pol;
      pb_2_pls       <= pb_debncd_2_pol and NOT pb_debncd_2_d1;
      --
      if (pb_2_pls = '1') then
         affirm_hold <= '1';
      elsif (step = '1') then
         affirm_hold <= '0';
      end if;
      --
    end if;
  end process;

  pr_gen_i : pr_gen
  port map
   (
    clk          => clk,                --in  std_logic;
    start        => start,              --in  std_logic;
    speed_select => (NOT speed_select), --in  std_logic;
    prbs         => prbs,               --out std_logic_vector(2 downto 0);
    tag          => tag,                --out std_logic--active for one full cycle time
    step         => step                --out std_logic
   );

  tag_delay : process(clk)
  begin
    if rising_edge(clk) then
       if (step = '1') then
          tag_d <= tag; --tag goes into FIFO, tag_d flash display
      end if;
    end if;
  end process;
   
  fifo_i : fifo
  port map
   (
    clk      => clk,                  --in  std_logic;
    start    => start,                --in  std_logic;
    prbs_in  => prbs,                 --in  std_logic_vector(2 downto 0);
    tag_in   => tag,                  --in  std_logic;
    wr       => (step AND NOT tag_d), --in  std_logic; --flash display w/ tag_d
    --
    rd       => (rd_fifo OR affirm_pls), --in  std_logic;
    level    => level,                --out std_logic_vector(4 downto 0);
    prbs_out => prbs_out,             --out std_logic_vector(2 downto 0);
    tag_out  => tag_out               --out std_logic
   );
   
  display_drive_i : display_drive
  port map
   (
    clk          => clk,             --in   std_logic;
    fifo_input   => prbs,            --in   std_logic_vector(2 downto 0);
    tag          => tag_d,           --in   std_logic; -- flash display
    end_game     => end_game,        --in   std_logic;
    fifo_output  => prbs_out,        --in   std_logic_vector(2 downto 0);
    fifo_level   => level,           --in   std_logic_vector(4 downto 0);
    point_count  => point_count_std, --in   std_logic_vector(3 downto 0);
    --
    seven_seg_1  => seven_seg_1,     --out  std_logic_vector(7 downto 0);
    seven_seg_2  => seven_seg_2,     --out  std_logic_vector(7 downto 0);
    seven_seg_3  => seven_seg_3,     --out  std_logic_vector(7 downto 0);
    seven_seg_4  => seven_seg_4,     --out  std_logic_vector(7 downto 0);
    en1          => en1,             --out  std_logic;
    en2          => en2,             --out  std_logic;
    en3          => en3,             --out  std_logic;
    en4          => en4              --out  std_logic 
   );

  next_samp   <= polarity XOR pb_debncd_1;
  affirm      <= polarity XOR pb_debncd_2;
  next_pls    <= pb_1_pls;
  affirm_pls  <= pb_2_pls AND (NOT affirm_hold);
   
  state_machine : process(clk)
  begin
    if rising_edge(clk) then
      case (fifo_state) is
        when idle  => if (affirm_pls = '1') then
                         if (tag_out = '1') then    fifo_state <= incr;
                         else                       fifo_state <= decr; 
                         end if;
                      elsif (next_pls = '1') then   fifo_state <= pop; 
                      elsif (step = '1') then 
--                      if (level = X"F") then      fifo_state <= full;
                        if (level = '1' & X"F") then fifo_state <= full;
                        elsif (    next_samp = '1'
                               AND affirm = '1'
                              ) then                fifo_state <= clear;
                        else                        fifo_state <= idle;
                        end if;
                      end if;
        when incr  =>                               fifo_state <= pop; 
        when decr  =>                               fifo_state <= pop; 
        when pop   =>                               fifo_state <= idle; 
        when full  =>  if (    next_samp = '1'
                           AND affirm = '1'
                          ) then                    fifo_state <= clear; 
                       end if;
        when clear =>                               fifo_state <= idle; 
        --
        when others => fifo_state <= idle;
      end case;
    end if;
  end process;

  state_machine_outputs : process(clk)
  begin
    if rising_edge(clk) then
      start          <= '0';
      point_cnt_clr  <= '0';
      point_cnt_incr <= '0';
      point_cnt_decr <= '0';
      rd_fifo        <= '0';
      end_game       <= '0';
      --
      case (fifo_state) is
        when idle  => null;
        when incr  => point_cnt_incr <= '1';
        when decr  => point_cnt_decr <= '1';
        when pop   => rd_fifo        <= '1';
        when full  => point_cnt_clr  <= '1';
                      end_game       <= '1';
        when clear => start          <= '1';
                      point_cnt_clr  <= '1';
        --
        when others => null;
      end case;
    end if;
  end process;
  
  point_count_i : process(clk)
  begin
    if rising_edge(clk) then
      if (point_cnt_clr = '1') then
         point_count <= X"0";
      elsif (    point_count /= X"F"
             AND point_cnt_incr = '1'
            ) then
         point_count <= point_count + 1;
      elsif (    point_count /= X"0"
             AND point_cnt_decr = '1'
            ) then
         point_count <= point_count - 1;
      end if;
    end if;
  end process;
  
  point_count_std <= std_logic_vector(point_count);

end architecture Behavioral;

