
-- Project: match_game
--
-- game to match sequences of numbers.

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_MISC.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.NUMERIC_STD.all;

use work.projects_pkg.all;

entity match_game is
  port 
   (
    clk          : in  std_logic;
    pb_1         : in  std_logic;
    pb_2         : in  std_logic;
    speed_select : in  std_logic_vector(1 downto 0);
    --
    seven_seg_1  : out std_logic_vector(7 downto 0);
    seven_seg_2  : out std_logic_vector(7 downto 0);
    seven_seg_3  : out std_logic_vector(7 downto 0);
    seven_seg_4  : out std_logic_vector(7 downto 0);
    en1          : out std_logic;
    en2          : out std_logic;
    en3          : out std_logic;
    en4          : out std_logic;
    led_1        : out std_logic;
    led_2        : out std_logic;
    -- unused LEDs
    led       : out  std_logic_vector(9 downto 2)
   );
end entity;

architecture Behavioral of match_game is

   -- component declarations
  component debounce is
  port 
   (
    clk          : in   std_logic;
    pb_in        : in   std_logic;
    --
    pb_debounced : out  std_logic
   );
   end component;

  component seg_drive
  port 
   (
    clk          : in  std_logic;
    mem_dat_1    : in  std_logic_vector(7 downto 0);
    mem_dat_2    : in  std_logic_vector(7 downto 0);
    --
    seven_seg_1  : out std_logic_vector(7 downto 0);
    seven_seg_2  : out std_logic_vector(7 downto 0);
    seven_seg_3  : out std_logic_vector(7 downto 0);
    seven_seg_4  : out std_logic_vector(7 downto 0);
    en1          : out std_logic;
    en2          : out std_logic;
    en3          : out std_logic;
    en4          : out std_logic 
   );
  end component;
  
  type state_type is (
                       idle,
                       clear,
                       decide,
                       good,
                       bad_affirm,
                       miss,
                       pass,
                       wait_step,
                       blink_display --new
                     );
  signal state : state_type;
  
  -- signal declarations
  signal pb_debncd_1    : std_logic;
  signal pb_debncd_1_d1 : std_logic;
  signal pb_debncd_2    : std_logic;
  signal pb_debncd_2_d1 : std_logic;
  signal go           : std_logic;
  signal step         : std_logic;
  signal affirm       : std_logic;
  signal factor1      : std_logic;
  signal factor2      : std_logic;
  signal factor3      : std_logic;
  signal match        : std_logic;
  signal step_load    : unsigned(28 downto 0);
  signal step_cnt     : unsigned(28 downto 0); -- 2 seconds: 0xBEBC200 at 100MHz, 0x5F5E100 at 50MHz
  signal prs1         : std_logic_vector(9 downto 1);
  signal prs2         : std_logic_vector(9 downto 1);
  signal prs3         : std_logic_vector(9 downto 1);
  signal next_char_1  : std_logic_vector(3 downto 0);
  signal next_char_2  : std_logic_vector(3 downto 0);
  signal next_char_3  : std_logic_vector(3 downto 0);
  signal match_reg_a1 : std_logic_vector(3 downto 0);
  signal match_reg_b1 : std_logic_vector(3 downto 0);
  signal match_reg_c1 : std_logic_vector(3 downto 0);
  signal match_reg_a2 : std_logic_vector(3 downto 0);
  signal match_reg_b2 : std_logic_vector(3 downto 0);
  signal match_reg_c2 : std_logic_vector(3 downto 0);
  signal point_cnt    : unsigned(3 downto 0); -- counts 0 to 9
  signal seg_byte_1   : std_logic_vector(7 downto 0);
  signal seg_byte_2   : std_logic_vector(7 downto 0);
  signal blink_val    : std_logic_vector(3 downto 0);
  signal go_pushed    : std_logic;
   
begin

  --constants from the package.
  step_load <= period1 when speed_select = "00" else 
               period2 when speed_select = "01" else 
               period3 when speed_select = "10" else 
               period4;                  --"11"

  debounce_1 : debounce
  port map
   (
    clk          => clk,         --in   std_logic;
    pb_in        => pb_1,        --in   std_logic;
    --
    pb_debounced => pb_debncd_1  --out  std_logic
   );

  debounce_2 : debounce
  port map
   (
    clk          => clk,         --in   std_logic;
    pb_in        => pb_2,        --in   std_logic;
    --
    pb_debounced => pb_debncd_2  --out  std_logic
   );

  debounce_proc : process(clk)
  begin
    if rising_edge(clk) then
      pb_debncd_1_d1 <= pb_debncd_1;
      if (pb_debncd_1 = '0' and pb_debncd_1_d1 = '1') then 
         go_pushed <= '1';
         go        <= not go;
      else
         go_pushed <= '0';
      end if;
      --
      pb_debncd_2_d1 <= pb_debncd_2;
      if (pb_debncd_2 = '0' and pb_debncd_2_d1 = '1') then 
         affirm <= '1';
      elsif (step = '1') then
         affirm <= '0';
      end if;
    end if;
  end process;

  timer : process(clk)
  begin
     if rising_edge(clk) then
        if (step_cnt = '0' & X"0000000") then
           step_cnt <= step_load;
           step <= '1';
        else
           step_cnt <= step_cnt - 1;
           step <= '0';
        end if;
     end if;
  end process;

  led_1 <= go;
  led_2 <= affirm;

  prs_process : process(clk)
  begin
     if rising_edge(clk) then
        if (state = clear) then --load seeds
           prs1 <= "000000001" OR std_logic_vector(step_cnt(8 downto 0));
           prs2 <= "000000001" OR std_logic_vector(step_cnt(17 downto 9));
           prs3 <= "000000001" OR std_logic_vector(step_cnt(26 downto 18));
        else
           prs1 <= prs1(1) & prs1(9 downto 7) & (prs1(6) xor prs1(1)) & prs1(5 downto 2);
           prs2 <= prs2(1) & prs2(9 downto 7) & (prs2(6) xor prs2(1)) & prs2(5 downto 2);
           prs3 <= prs3(1) & prs3(9 downto 7) & (prs3(6) xor prs3(1)) & prs3(5 downto 2);
        end if;
     end if;
  end process;

  next_char_1 <= X"1" when     (prs1(4 downto 1) >= X"0")
                           and (prs1(4 downto 1) <= X"1") else
                 X"2" when     (prs1(4 downto 1) >= X"2")
                           and (prs1(4 downto 1) <= X"3") else
                 X"3" when     (prs1(4 downto 1) >= X"4")
                           and (prs1(4 downto 1) <= X"5") else
                 X"4" when     (prs1(4 downto 1) >= X"6")
                           and (prs1(4 downto 1) <= X"7") else
                 X"5" when     (prs1(4 downto 1) >= X"8")
                           and (prs1(4 downto 1) <= X"9") else
                 X"6" when     (prs1(4 downto 1) >= X"A")
                           and (prs1(4 downto 1) <= X"B") else
                 X"7" when     (prs1(4 downto 1) >= X"C")
                           and (prs1(4 downto 1) <= X"D") else
                 X"8";

  next_char_2 <= X"1" when     (prs2(4 downto 1) >= X"0")
                           and (prs2(4 downto 1) <= X"1") else
                 X"2" when     (prs2(4 downto 1) >= X"2")
                           and (prs2(4 downto 1) <= X"3") else
                 X"3" when     (prs2(4 downto 1) >= X"4")
                           and (prs2(4 downto 1) <= X"5") else
                 X"4" when     (prs2(4 downto 1) >= X"6")
                           and (prs2(4 downto 1) <= X"7") else
                 X"5" when     (prs2(4 downto 1) >= X"8")
                           and (prs2(4 downto 1) <= X"9") else
                 X"6" when     (prs2(4 downto 1) >= X"A")
                           and (prs2(4 downto 1) <= X"B") else
                 X"7" when     (prs2(4 downto 1) >= X"C")
                           and (prs2(4 downto 1) <= X"D") else
                 X"8";

  next_char_3 <= X"1" when     (prs3(4 downto 1) >= X"0")
                           and (prs3(4 downto 1) <= X"1") else
                 X"2" when     (prs3(4 downto 1) >= X"2")
                           and (prs3(4 downto 1) <= X"3") else
                 X"3" when     (prs3(4 downto 1) >= X"4")
                           and (prs3(4 downto 1) <= X"5") else
                 X"4" when     (prs3(4 downto 1) >= X"6")
                           and (prs3(4 downto 1) <= X"7") else
                 X"5" when     (prs3(4 downto 1) >= X"8")
                           and (prs3(4 downto 1) <= X"9") else
                 X"6" when     (prs3(4 downto 1) >= X"A")
                           and (prs3(4 downto 1) <= X"B") else
                 X"7" when     (prs3(4 downto 1) >= X"C")
                           and (prs3(4 downto 1) <= X"D") else
                 X"8";

   
  match_registers : process(clk)
  begin
     if rising_edge(clk) then
        if (step = '1') then
           match_reg_a1 <= next_char_1;
           match_reg_b1 <= next_char_2;
           match_reg_c1 <= next_char_3;
           --
           match_reg_a2 <= match_reg_a1;
           match_reg_b2 <= match_reg_b1;
           match_reg_c2 <= match_reg_c1;
        end if;
     end if;
  end process;
  
  factor1 <= '1' when (   match_reg_a2 = match_reg_a1
                       or match_reg_a2 = match_reg_b1
                       or match_reg_a2 = match_reg_c1
                      ) else '0';

  factor2 <= '1' when (   match_reg_b2 = match_reg_a1
                       or match_reg_b2 = match_reg_b1
                       or match_reg_b2 = match_reg_c1
                      ) else '0';

  factor3 <= '1' when (   match_reg_c2 = match_reg_a1
                       or match_reg_c2 = match_reg_b1
                       or match_reg_c2 = match_reg_c1
                      ) else '0';
  
  match <= '1' when (   (factor1 = '1' and factor2 = '1')
                     or (factor1 = '1' and factor3 = '1')
                     or (factor2 = '1' and factor3 = '1')
                     ) else '0';

  state_machine : process(clk)
  begin
     if rising_edge(clk) then
        case (state) is 
          when idle       => if (go = '1') then        state <= clear;  end if;
          when clear      =>                           state <= decide;
          when decide     => if (affirm = '1') then
                               if (match = '1') then   state <= good;
                               else                    state <= bad_affirm; end if;
                             elsif (step = '1') then
                               if (match = '1') then   state <= miss;
                               else                    state <= pass; end if;
                             end if;
--        when good       =>                           state <= wait_step;
          -- note that the point count increments from 8 to 9 here.
          when good       => if (point_cnt = X"8") then state <= blink_display;
                             else                      state <= wait_step; end if;
          when bad_affirm =>                           state <= wait_step;
          when wait_step  => if (step = '1') then
                               if (go = '1') then      state <= decide;
                               else                    state <= idle; end if;
                             end if;
          when miss       => if (go = '1') then        state <= decide;
                             else                      state <= idle; end if;
          when pass       => if (go = '1') then        state <= decide;
                             else                      state <= idle; end if;
          when blink_display => if (go_pushed = '1') then state <= decide; end if;
          when others     => state <= idle;
        end case;
     end if;
  end process;
  
  point_counter : process(clk)
  begin
     if rising_edge(clk) then
        if (state = clear) then
           point_cnt  <= X"0";
        elsif (    point_cnt /= X"9"
               and state = good
              ) then
           point_cnt  <= point_cnt + 1;
        elsif (    point_cnt /= 0
               and (   state = bad_affirm
                    or state = miss
                   )
              ) then
           point_cnt  <= point_cnt - 1;
        end if;
     end if;
  end process;
  
  blink_val <= step_cnt(24) & step_cnt(24) & '0' & (NOT step_cnt(24));
   
  seg_byte_1 <= blink_val    & blink_val    when (state = blink_display) else
                match_reg_a1 & match_reg_b1 when go = '1' 
                                            else X"00";
  
  seg_byte_2 <= blink_val    & blink_val                   when (state = blink_display) else
                match_reg_c1 & std_logic_vector(point_cnt) when go = '1' 
                                                           else X"00";
  
  seg_drive_i : seg_drive
  port map
   (
    clk          => clk,            --in   std_logic;
    mem_dat_1    => seg_byte_1,     --in   std_logic_vector(7 downto 0);
    mem_dat_2    => seg_byte_2,     --in   std_logic_vector(7 downto 0);
    --                              
    seven_seg_1  => seven_seg_1,    --out  std_logic_vector(7 downto 0);
    seven_seg_2  => seven_seg_2,    --out  std_logic_vector(7 downto 0);
    seven_seg_3  => seven_seg_3,    --out  std_logic_vector(7 downto 0);
    seven_seg_4  => seven_seg_4,    --out  std_logic_vector(7 downto 0);
    en1          => en1,            --out  std_logic;
    en2          => en2,            --out  std_logic;
    en3          => en3,            --out  std_logic;
    en4          => en4             --out  std_logic 
   );

end architecture Behavioral;

